#include "DeviceManager.h"

DeviceManager::DeviceManager()
{
}

DeviceManager* DeviceManager::CreateDevice(int width,int height,bool fullscreen,HWND handle)
{
	HRESULT hr=0;

	DXGI_SWAP_CHAIN_DESC sd;
	ZeroMemory( &sd, sizeof( sd ) );
	sd.BufferCount = 1;
	sd.BufferDesc.Width = width;
	sd.BufferDesc.Height = height;
	sd.BufferDesc.Format = DXGI_FORMAT_R8G8B8A8_UNORM;
	sd.BufferDesc.RefreshRate.Numerator = 60;
	sd.BufferDesc.RefreshRate.Denominator = 1;
	sd.BufferUsage = DXGI_USAGE_RENDER_TARGET_OUTPUT;
	sd.OutputWindow = handle;
	sd.SampleDesc.Count = 1;
	sd.SampleDesc.Quality = 0;
	sd.Windowed = TRUE;


	DeviceManager* d=new DeviceManager();
	d->isFullscreen=fullscreen;
	d->width=width;
	d->height=height;

	D3D_FEATURE_LEVEL  FeatureLevelsRequested[3]={D3D_FEATURE_LEVEL_11_0,D3D_FEATURE_LEVEL_10_1,D3D_FEATURE_LEVEL_10_0};
	UINT               numLevelsRequested = 3;
	D3D_FEATURE_LEVEL  FeatureLevelsSupported[6];

	if( FAILED (hr = D3D11CreateDeviceAndSwapChain( NULL, 
					D3D_DRIVER_TYPE_HARDWARE, 
					NULL, 
					0,
					FeatureLevelsRequested, 
					numLevelsRequested, 
					D3D11_SDK_VERSION, 
					&sd, 
					&d->swapChain, 
					&d->device, 
					FeatureLevelsSupported,
					&d->context)))
	{
		return NULL;
	}

	d->CreateTargetView();
	d->CreateDepthStencilView(width,height);

	d->context->OMSetRenderTargets( 1, &d->renderTargetView, d->renderTargetDepth);

	d->SetViewport(width,height);


	D3D11_DEPTH_STENCIL_DESC descD=D3D11_DEPTH_STENCIL_DESC();
	descD.DepthEnable=true;
	descD.DepthWriteMask=D3D11_DEPTH_WRITE_MASK_ALL;
	descD.DepthFunc=D3D11_COMPARISON_LESS_EQUAL;

	D3D11_RASTERIZER_DESC rastD=D3D11_RASTERIZER_DESC();
	rastD.CullMode=D3D11_CULL_BACK;
	rastD.FillMode=D3D11_FILL_SOLID;
		
	D3D11_SAMPLER_DESC sampD=D3D11_SAMPLER_DESC();
	sampD.AddressU=D3D11_TEXTURE_ADDRESS_WRAP;
	sampD.AddressV=D3D11_TEXTURE_ADDRESS_WRAP;
	sampD.AddressW=D3D11_TEXTURE_ADDRESS_WRAP;
	sampD.Filter=D3D11_FILTER_MIN_MAG_MIP_LINEAR;
		
    D3D11_BLEND_DESC blenD=D3D11_BLEND_DESC();
    blenD.IndependentBlendEnable = false;
    blenD.RenderTarget[0].BlendEnable =  false;
    blenD.RenderTarget[0].RenderTargetWriteMask = D3D11_COLOR_WRITE_ENABLE_ALL;
	

	d->device->CreateRasterizerState(&rastD,&d->rasterState);
	d->device->CreateDepthStencilState(&descD,&d->depthState);
	d->device->CreateSamplerState(&sampD,&d->samplerState);
    d->device->CreateBlendState(&blenD, &d->blendState );

	
	d->context->RSSetState(d->rasterState);
	d->context->OMSetDepthStencilState(d->depthState,0);
	d->context->OMSetBlendState( d->blendState, 0, 0xffffffff );

	ID3D11SamplerState* samps[1]={d->samplerState};
	d->context->PSSetSamplers(0,1,samps);
	d->context->DSSetSamplers(0,1,samps);
	return d;
}




bool DeviceManager::Resize(int width,int height)
{
	if(isFullscreen)return true;

	this->width=width;
	this->height=height;

	renderTargetView->Release();
	renderTargetDepth->Release();

	HRESULT hr= swapChain->ResizeBuffers(1,width,height,DXGI_FORMAT_R8G8B8A8_UNORM,DXGI_SWAP_CHAIN_FLAG_ALLOW_MODE_SWITCH);
	CreateTargetView();
	CreateDepthStencilView(width,height);
	SetViewport(width,height);

	context->OMSetRenderTargets( 1, &renderTargetView, renderTargetDepth);
	return true;
}


bool DeviceManager::CreateTargetView()
{
	// Crea il render target
	// Crea il render target
    ID3D11Texture2D *pBackBuffer;
    if( FAILED( swapChain->GetBuffer( 0, __uuidof( ID3D11Texture2D ), (LPVOID*)&pBackBuffer ) ) )
        return false;
	HRESULT hr = device->CreateRenderTargetView( pBackBuffer, NULL, &renderTargetView );
    pBackBuffer->Release();
    if( FAILED( hr ) )
        return false;
	else
		return true;
}

bool DeviceManager::CreateDepthStencilView(int width,int height)
{
	// crea depth stencil
	//create the depth stencil
    D3D11_TEXTURE2D_DESC descDepth;
    descDepth.Width = width;
    descDepth.Height = height;
    descDepth.MipLevels = 1;
    descDepth.ArraySize = 1;
	descDepth.Format = DXGI_FORMAT_D32_FLOAT;
    descDepth.SampleDesc.Count = 1;
    descDepth.SampleDesc.Quality = 0;
    descDepth.Usage = D3D11_USAGE_DEFAULT;
    descDepth.BindFlags = D3D11_BIND_DEPTH_STENCIL;
    descDepth.CPUAccessFlags = 0;
    descDepth.MiscFlags = 0;
    
	ID3D11Texture2D *pDepthBuffer;

	HRESULT hr = device->CreateTexture2D( &descDepth, NULL, &pDepthBuffer );
    if(hr!=S_OK)    
		return false;

    D3D11_DEPTH_STENCIL_VIEW_DESC descDSV;
	descDSV.Flags=0;
	descDSV.Format = DXGI_FORMAT_D32_FLOAT;
	descDSV.ViewDimension= D3D11_DSV_DIMENSION_TEXTURE2D;
	descDSV.Texture2D.MipSlice=0;
    hr= device->CreateDepthStencilView( pDepthBuffer, &descDSV, &renderTargetDepth );
    pDepthBuffer->Release();    

	if(hr!=S_OK)    
		return false;

	return true;
}

void DeviceManager::SetViewport(int width, int height)
{
	//imposta il viewport, l'area visibile
	//imposta il viewport
    D3D11_VIEWPORT vp;
    vp.Width = (float)width;
    vp.Height = (float)height;
    vp.MinDepth = 0.0f;
    vp.MaxDepth = 1.0f;
    vp.TopLeftX = 0;
    vp.TopLeftY = 0;
    context->RSSetViewports( 1, &vp );

}

void DeviceManager::RestoreTarget()
{
    context->OMSetRenderTargets( 1, &renderTargetView, renderTargetDepth);
	SetViewport(width,height);
}

void DeviceManager::Clear(float r,float g,float b,float a)
{
	float ClearColor[4] = { r, g, b, a};
	context->ClearRenderTargetView(renderTargetView,ClearColor);
	context->ClearDepthStencilView(renderTargetDepth,D3D11_CLEAR_DEPTH,1,0);
}


void DeviceManager::Present()
{
	swapChain->Present(0,0);
}

void DeviceManager::Release()
{
	context->ClearState();
	SAFE_RELEASE(context);
	SAFE_RELEASE(renderTargetView);
	SAFE_RELEASE(renderTargetDepth);
	SAFE_RELEASE(swapChain);
	SAFE_RELEASE(rasterState);
	SAFE_RELEASE(depthState);
	SAFE_RELEASE(samplerState);
	SAFE_RELEASE(device);
}

int DeviceManager::GetWidth()
{
	return width;
}

int DeviceManager::GetHeight()
{
	return height;
}