#include "EffectManager.h"

EffectManager::EffectManager()
{
	
	vertexByteCode=NULL;
	vertexShader=NULL;
	pixelShader=NULL;
	geometryShader=NULL;
	hullShader=NULL;
	domainShader=NULL;
}

EffectManager* EffectManager::Init(DeviceManager* device,char* filename,char* vertexShaderEntry, char* pixelShaderEntry, char* geometryShaderEntry,char* domainShaderEntry,char* hullShaderEntry,bool model5)
{
	EffectManager* f=new EffectManager();

	f->devicePointer=device;
	if(vertexShaderEntry!=NULL)
	{
		if(!f->Compile(filename,vertexShaderEntry,Target_VertexShader,model5))
		{
			f->Release();
			delete f;
			return NULL;
		}
	}

	if(pixelShaderEntry!=NULL)
	{
		if(!f->Compile(filename,pixelShaderEntry,Target_PixelShader,model5))
		{
			f->Release();
			delete f;
			return NULL;
		}
	}

	if(geometryShaderEntry!=NULL)
	{
		if(!f->Compile(filename,geometryShaderEntry,Target_GeometryShader,model5))
		{
			f->Release();
			delete f;
			return NULL;
		}
	}

	if(domainShaderEntry!=NULL)
	{
		if(!f->Compile(filename,domainShaderEntry,Target_DomainShader,model5))
		{
			f->Release();
			delete f;
			return NULL;
		}
	}

	if(hullShaderEntry!=NULL)
	{
		if(!f->Compile(filename,hullShaderEntry,Target_HullShader,model5))
		{
			f->Release();
			delete f;
			return NULL;
		}
	}


	return f;
}

bool EffectManager::Compile(char* filename,char* entryPoint,ShaderTarget target,bool model5)
{

	char* model;

	switch(target)
	{
	case Target_VertexShader:
		if(model5)
			model="vs_5_0";
		else
			model="vs_4_0";
		break;

	case Target_PixelShader:
		if(model5)
			model="ps_5_0";
		else
			model="ps_4_0";
		break;

	case Target_GeometryShader:
		if(model5)
			model="gs_5_0";
		else
			model="gs_4_0";	
		break;
		
	case Target_HullShader:
		if(model5)
			model="hs_5_0";
		else
			model="hs_4_0";
		break;
		
	case Target_DomainShader:
		if(model5)
			model="ds_5_0";
		else
			model="ds_4_0";
		break;
	}

	ID3DBlob* code;
	ID3DBlob* error;
	
	HRESULT hr= D3DX11CompileFromFileA(filename,NULL,NULL,entryPoint,model,0,0,NULL,&code,&error,NULL);
	
	if(hr!=S_OK)
	{
		//errore
		if(error!=NULL)
		{
			MessageBoxA(NULL,(char*)error->GetBufferPointer(),"Compile Error",MB_ICONASTERISK);
			SAFE_RELEASE(error);
		}
		return false;
	}

	switch(target)
	{
	case Target_VertexShader:
		vertexByteCode=new byte[code->GetBufferSize()];
		vertexByteSize=code->GetBufferSize();
		memcpy_s(vertexByteCode,code->GetBufferSize(),code->GetBufferPointer(),code->GetBufferSize());
		hr=devicePointer->device->CreateVertexShader(code->GetBufferPointer(), code->GetBufferSize(),NULL,&vertexShader);
		break;

	case Target_PixelShader:
		hr=devicePointer->device->CreatePixelShader(code->GetBufferPointer(), code->GetBufferSize(),NULL,&pixelShader);
		break;

	case Target_GeometryShader:
		hr=devicePointer->device->CreateGeometryShader(code->GetBufferPointer(), code->GetBufferSize(),NULL,&geometryShader);
		break;
		
	case Target_HullShader:
		hr=devicePointer->device->CreateHullShader(code->GetBufferPointer(), code->GetBufferSize(),NULL,&hullShader);
		break;
		
	case Target_DomainShader:
		hr=devicePointer->device->CreateDomainShader(code->GetBufferPointer(), code->GetBufferSize(),NULL,&domainShader);

		break;
	}

	SAFE_RELEASE(code);
	SAFE_RELEASE(error);

	if(hr!=S_OK)
	{
		//errore
		return false;
	}

	return true;
}


LPVOID EffectManager::GetByteCodeBufferPointer()
{
	return vertexByteCode;
}

int EffectManager::GetByteCodeBufferSize()
{
	return vertexByteSize;
}

void EffectManager::Release()
{
	if(vertexByteCode!=0)
		delete[] vertexByteCode;

	SAFE_RELEASE(vertexShader);
	SAFE_RELEASE(pixelShader);
	SAFE_RELEASE(geometryShader);
	SAFE_RELEASE(hullShader);
	SAFE_RELEASE(domainShader);
}

void EffectManager::Apply()
{
	devicePointer->context->VSSetShader(vertexShader,NULL,0);
	devicePointer->context->PSSetShader(pixelShader,NULL,0);
	devicePointer->context->GSSetShader(geometryShader,NULL,0);
	devicePointer->context->HSSetShader(hullShader,NULL,0);
	devicePointer->context->DSSetShader(domainShader,NULL,0);
}