#include "TutorialApplication.h"

TutorialApplication::TutorialApplication(int width,int height,HWND handle, bool fullscreen):BaseApplication(width,height,handle, fullscreen)
{
	
}

bool TutorialApplication::Init()
{

	vertexBuffer=0;
	layout=0;
	constantBuffer=0;
	texture=0;
	sampler=0;

	effect=EffectManager::Init(manager,"shader.hlsl","VSMain","PSMain",NULL,NULL,NULL,false);

	if(effect==NULL)
		return false;

	constantBuffer=ConstantBuffer::Init(manager,64);
	if(constantBuffer==NULL)
		return false;

	D3D11_INPUT_ELEMENT_DESC desc[] =
    {
		{ "POSITION", 0,DXGI_FORMAT_R32G32B32_FLOAT , 0, 0,D3D11_INPUT_PER_VERTEX_DATA , 0 }, 
		{ "TEXCOORD", 0, DXGI_FORMAT_R32G32_FLOAT, 0, 12,D3D11_INPUT_PER_VERTEX_DATA , 0 }, 
    };
     
	HRESULT hr= manager->device->CreateInputLayout(desc,2,effect->GetByteCodeBufferPointer(),effect->GetByteCodeBufferSize(),&layout);
	if(hr!=S_OK)
		return false;

	D3D11_BUFFER_DESC bd;
	
	bd.Usage = D3D11_USAGE_DEFAULT;
    bd.ByteWidth = 20 * 36;
	bd.BindFlags = D3D11_BIND_VERTEX_BUFFER;
    bd.CPUAccessFlags = 0;
    bd.MiscFlags = 0;
	
	VERTEX vertices[36]=
	{
	
		{-1, -1, -1, 0, 1},
        {-1, 1, -1, 0, 0},
        {1, 1, -1, 1, 0},
        {1, 1, -1, 1, 0},
        {1, -1, -1, 1, 1},
        {-1, -1, -1, 0, 1},
        //'2
        {-1, -1, 1, 1, 1},
        {1, -1, 1, 0, 1},
        {1, 1, 1, 0, 0},
        {1, 1, 1, 0, 0},
        {-1, 1, 1, 1, 0},
        {-1, -1, 1, 1, 1},
        //'3
        {-1, -1, -1, 0, 1},
        {1, -1, -1, 1, 1},
        {1, -1, 1, 1, 0},
        {1, -1, 1, 1, 0},
        {-1, -1, 1, 0, 0},
        {-1, -1, -1, 0, 1},
        //'4
        {1, -1, -1, 0, 1},
        {1, 1, -1, 0, 0},
        {1, 1, 1, 1, 0},
        {1, 1, 1, 1, 0},
        {1, -1, 1, 1, 1},
        {1, -1, -1, 0, 1},
        //5
        {1, 1, -1, 1, 1},
        {-1, 1, -1, 0, 1},
        {-1, 1, 1, 0, 0},
        {-1, 1, 1, 0, 0},
        {1, 1, 1, 1, 0},
        {1, 1, -1, 1, 1},
        //6
        {-1, 1, -1, 1, 0},
        {-1, -1, -1, 1, 1},
        {-1, -1, 1, 0, 1},
        {-1, -1, 1, 0, 1},
        {-1, 1, 1, 0, 0},
        {-1, 1, -1, 1, 0}
	};
	
	D3D11_SUBRESOURCE_DATA data;
	data.pSysMem=vertices;
	hr = manager->device->CreateBuffer(&bd,&data,&vertexBuffer);
	if(hr!=S_OK)
		return false;


	hr=D3DX11CreateShaderResourceViewFromFileA(manager->device,"texture0.bmp",NULL,NULL,&texture,NULL);
	if(hr!=S_OK)
		return false;

	D3D11_SAMPLER_DESC sd;
	ZeroMemory(&sd,sizeof(sd));

	sd.AddressU=D3D11_TEXTURE_ADDRESS_WRAP;
	sd.AddressV=D3D11_TEXTURE_ADDRESS_WRAP;
	sd.AddressW=D3D11_TEXTURE_ADDRESS_WRAP;
	sd.Filter=D3D11_FILTER_MIN_MAG_MIP_LINEAR;
	hr=manager->device->CreateSamplerState(&sd,&sampler);
	if(hr!=S_OK)
		return false;

	return true;
}

void TutorialApplication::Release()
{
	SAFE_RELEASE(vertexBuffer);
	SAFE_RELEASE(layout);
	SAFE_RELEASE(constantBuffer);
	SAFE_RELEASE(texture);
	SAFE_RELEASE(sampler);
	BaseApplication::Release();
}

void TutorialApplication::Reset(int width,int height)
{
	manager->Resize(width,height);
}

void TutorialApplication::DoCommand(int key)
{
	
}

void TutorialApplication::Render()
{
	XMMATRIX proj=XMMatrixPerspectiveFovLH(3.14F/3.0F,1,1,1000);
	XMMATRIX view=XMMatrixLookAtLH(XMVectorSet(0,30,50,0),XMVectorSet(0,0,0,0),XMVectorSet(0,1,0,0));
	XMMATRIX world=XMMatrixScaling(10,10,10)* XMMatrixRotationY(GetTickCount()/1000.0F);
	XMMATRIX tot=world*view*proj;
	constantBuffer->Write(&tot,64);

	manager->Clear(0,0,1,0);
	ID3D11Buffer* cbuffers[1] = { constantBuffer->GetBufferPointer()};
	manager->context->VSSetConstantBuffers(0,1,cbuffers);

	ID3D11ShaderResourceView* views[]={texture};
	manager->context->PSSetShaderResources(0,1,views);

	ID3D11SamplerState* samplers[]={sampler};
	manager->context->PSSetSamplers(0,1,samplers);

	effect->Apply();
	
	manager->context->IASetInputLayout(layout);
	manager->context->IASetPrimitiveTopology(D3D11_PRIMITIVE_TOPOLOGY_TRIANGLELIST);
	UINT strides[]={20};
	UINT offsets[]={0};
	ID3D11Buffer* buffers[]= { vertexBuffer };
	manager->context->IASetVertexBuffers(0,1,buffers,strides,offsets);
	manager->context->Draw(36,0);

	manager->Present();
}